@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2018 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -26,18 +29,20 @@
 #include "jcr.h"
 
 
-#ifdef HAVE_CONIO
+#if defined(HAVE_CONIO)
 #include "conio.h"
 //#define CONIO_FIX 1
-#else
+#else /* defined(HAVE_READLINE) || "DUMB" */
 #define con_init(x)
 #define con_term()
 #define con_set_zed_keys();
-#define trapctlc()
-#define clrbrk()
-#define usrbrk() 0
 #endif
 
+void trapctlc();
+void clrbrk();
+int usrbrk();
+static int brkflg = 0;              /* set on user break */
+
 #if defined(HAVE_WIN32)
 #define isatty(fd) (fd==0)
 #endif
@@ -47,8 +52,7 @@
 //extern int rl_catch_signals;
 
 /* Imported functions */
-int authenticate_director(JCR *jcr, DIRRES *director, CONRES *cons);
-extern bool parse_cons_config(CONFIG *config, const char *configfile, int exit_code);
+int authenticate_director(BSOCK *dir, DIRRES *director, CONRES *cons);
 
 /* Forward referenced functions */
 static void terminate_console(int sig);
@@ -71,6 +75,7 @@ static DIRRES *dir = NULL;
 static CONRES *cons = NULL;
 static FILE *output = stdout;
 static bool teeout = false;               /* output to output and stdout */
+static bool teein = false;                /* input to output and stdout */
 static bool stop = false;
 static bool no_conio = false;
 static int timeout = 0;
@@ -87,12 +92,15 @@ static int versioncmd(FILE *input, BSOCK *UA_sock);
 static int inputcmd(FILE *input, BSOCK *UA_sock);
 static int outputcmd(FILE *input, BSOCK *UA_sock);
 static int teecmd(FILE *input, BSOCK *UA_sock);
+static int teeallcmd(FILE *input, BSOCK *UA_sock);
 static int quitcmd(FILE *input, BSOCK *UA_sock);
 static int helpcmd(FILE *input, BSOCK *UA_sock);
 static int echocmd(FILE *input, BSOCK *UA_sock);
 static int timecmd(FILE *input, BSOCK *UA_sock);
 static int sleepcmd(FILE *input, BSOCK *UA_sock);
 static int execcmd(FILE *input, BSOCK *UA_sock);
+static int putfilecmd(FILE *input, BSOCK *UA_sock);
+
 #ifdef HAVE_READLINE
 static int eolcmd(FILE *input, BSOCK *UA_sock);
 
@@ -111,10 +119,12 @@ static void usage()
 {
    fprintf(stderr, _(
 PROG_COPYRIGHT
-"\nVersion: " VERSION " (" BDATE ") %s %s %s\n\n"
+"\n%sVersion: " VERSION " (" BDATE ") %s %s %s\n\n"
 "Usage: bconsole [-s] [-c config_file] [-d debug_level]\n"
 "       -D <dir>    select a Director\n"
 "       -l          list Directors defined\n"
+"       -L          list Consoles defined\n"
+"       -C <cons>   select a console\n"
 "       -c <file>   set configuration file to file\n"
 "       -d <nn>     set debug level to <nn>\n"
 "       -dt         print timestamp in debug output\n"
@@ -123,7 +133,7 @@ PROG_COPYRIGHT
 "       -u <nn>     set command execution timeout to <nn> seconds\n"
 "       -t          test - read configuration and exit\n"
 "       -?          print this message.\n"
-"\n"), 2000, HOST_OS, DISTNAME, DISTVER);
+"\n"), 2000, BDEMO, HOST_OS, DISTNAME, DISTVER);
 }
 
 
@@ -167,12 +177,14 @@ static struct cmdstruct commands[] = {
  { N_("output"),     outputcmd,    _("output to file")},
  { N_("quit"),       quitcmd,      _("quit")},
  { N_("tee"),        teecmd,       _("output to file and terminal")},
+ { N_("tall"),       teeallcmd,    _("output everything to file and terminal (tee all)")},
  { N_("sleep"),      sleepcmd,     _("sleep specified time")},
  { N_("time"),       timecmd,      _("print current time")},
  { N_("version"),    versioncmd,   _("print Console's version")},
  { N_("echo"),       echocmd,      _("echo command string")},
  { N_("exec"),       execcmd,      _("execute an external command")},
  { N_("exit"),       quitcmd,      _("exit = quit")},
+ { N_("putfile"),    putfilecmd,   _("send a file to the director")},
  { N_("zed_keys"),   zed_keyscmd,  _("zed_keys = use zed keys instead of bash keys")},
  { N_("help"),       helpcmd,      _("help listing")},
 #ifdef HAVE_READLINE
@@ -217,6 +229,38 @@ static int do_a_command(FILE *input, BSOCK *UA_sock)
    return stat;
 }
 
+/* When getting .api command, we can ignore some signals, so we set
+ * api_mode=true
+ */
+static bool api_mode=false;
+
+static bool ignore_signal(int stat, BSOCK *s)
+{
+   /* Not in API mode */
+   if (!api_mode) {
+      return false;
+   }
+
+   /* not a signal */
+   if (stat != -1) {
+      return false;
+   }
+
+   /* List signal that should not stop the read loop */
+   Dmsg1(100, "Got signal %s\n", bnet_sig_to_ascii(s->msglen));
+   switch(s->msglen) {
+   case BNET_CMD_BEGIN:
+   case BNET_CMD_FAILED:        /* might want to print **ERROR** */
+   case BNET_CMD_OK:            /* might want to print **OK** */
+   case BNET_MSGS_PENDING:
+      return true;
+   default:
+      break;
+   }
+
+   /* The signal should break the read loop */
+   return false;
+}
 
 static void read_and_process_input(FILE *input, BSOCK *UA_sock)
 {
@@ -235,7 +279,7 @@ static void read_and_process_input(FILE *input, BSOCK *UA_sock)
       }
       if (tty_input) {
          stat = get_cmd(input, prompt, UA_sock, 30);
-         if (usrbrk() == 1) {
+         if (usrbrk() >= 1) {
             clrbrk();
          }
          if (usrbrk()) {
@@ -243,11 +287,10 @@ static void read_and_process_input(FILE *input, BSOCK *UA_sock)
          }
       } else {
          /* Reading input from a file */
-         int len = sizeof_pool_memory(UA_sock->msg) - 1;
          if (usrbrk()) {
             break;
          }
-         if (fgets(UA_sock->msg, len, input) == NULL) {
+         if (bfgets(UA_sock->msg, input) == NULL) {
             stat = -1;
          } else {
             sendit(UA_sock->msg);  /* echo to terminal */
@@ -283,11 +326,23 @@ static void read_and_process_input(FILE *input, BSOCK *UA_sock)
          }
          stop_bsock_timer(tid);
       }
-      if (strcmp(UA_sock->msg, ".quit") == 0 || strcmp(UA_sock->msg, ".exit") == 0) {
+      if (strncasecmp(UA_sock->msg, ".api", 4) == 0) {
+         api_mode = true;
+      }
+      if (strcasecmp(UA_sock->msg, ".quit") == 0 || strcasecmp(UA_sock->msg, ".exit") == 0) {
          break;
       }
       tid = start_bsock_timer(UA_sock, timeout);
-      while ((stat = UA_sock->recv()) >= 0) {
+      while (1) {
+         stat = UA_sock->recv();
+         if (ignore_signal(stat, UA_sock)) {
+            continue;
+         }
+
+         if (stat < 0) {
+            break;
+         }
+
          if (at_prompt) {
             if (!stop) {
                sendit("\n");
@@ -314,7 +369,7 @@ static void read_and_process_input(FILE *input, BSOCK *UA_sock)
          if (UA_sock->msglen == BNET_SUB_PROMPT) {
             at_prompt = true;
          }
-         Dmsg1(100, "Got poll %s\n", bnet_sig_to_ascii(UA_sock));
+         Dmsg1(100, "Got poll %s\n", bnet_sig_to_ascii(UA_sock->msglen));
       }
    }
 }
@@ -448,12 +503,12 @@ static void match_kw(regex_t *preg, const char *what, int len, POOLMEM **buf)
 {
    int rc, size;
    int nmatch=20;
-   regmatch_t pmatch[20];
+   regmatch_t pmatch[nmatch];
 
    if (len <= 0) {
       return;
    }
-   rc = regexec(preg, what, nmatch, pmatch, 0);
+   rc = regexec(preg, what, nmatch, pmatch,  0);
    if (rc == 0) {
 #if 0
       Pmsg1(0, "\n\n%s\n0123456789012345678901234567890123456789\n        10         20         30\n", what);
@@ -599,7 +654,10 @@ static struct cpl_keywords_t cpl_keywords[] = {
    {"m",          ".ls"            },
    {"unmark",     ".lsmark"        },
    {"catalog=",   ".catalogs"      },
-   {"actiononpurge=", ".actiononpurge" }
+   {"actiononpurge=", ".actiononpurge" },
+   {"tags=",      ".tags"          },
+   {"recylepool=", ".pool"         },
+   {"allfrompool=",".pool"         }
 };
 #define key_size ((int)(sizeof(cpl_keywords)/sizeof(struct cpl_keywords_t)))
 
@@ -713,6 +771,14 @@ get_cmd(FILE *input, const char *prompt, BSOCK *sock, int sec)
    }
    if (command != line && isatty(fileno(input))) {
       senditf("%s%s\n", prompt, command);
+
+   } else {
+      /* Send the intput to the output file if needed */
+      if (teein && output != stdout) {
+         fputs(prompt, output);
+         fputs(command, output);
+         fputs("\n", output);
+      }
    }
 
    sock->msglen = pm_strcpy(&sock->msg, command);
@@ -753,15 +819,8 @@ wait_for_data(int fd, int sec)
 #if defined(HAVE_WIN32)
    return 1;
 #else
-   fd_set fdset;
-   struct timeval tv;
-
-   tv.tv_sec = sec;
-   tv.tv_usec = 0;
    for ( ;; ) {
-      FD_ZERO(&fdset);
-      FD_SET((unsigned)fd, &fdset);
-      switch(select(fd + 1, &fdset, NULL, NULL, &tv)) {
+      switch(fd_wait_data(fd, WAIT_READ, sec, 0)) {
       case 0:                         /* timeout */
          return 0;
       case -1:
@@ -829,11 +888,45 @@ again:
    }
    strip_trailing_junk(sock->msg);
    sock->msglen = strlen(sock->msg);
+
+   /* Send input to log file if needed */
+   if (teein && output != stdout) {
+      fputs(sock->msg, output);
+      fputs("\n", output);
+   }
+
    return 1;
 }
 
 #endif /* ! HAVE_READLINE */
 
+/* Routine to return true if user types break */
+int usrbrk()
+{
+   return brkflg;
+}
+
+/* Clear break flag */
+void clrbrk()
+{
+   brkflg = 0;
+}
+
+/* Interrupt caught here */
+static void sigintcatcher(int sig)
+{
+   brkflg++;
+   if (brkflg > 3) {
+      terminate_console(sig);
+   }
+   signal(SIGINT, sigintcatcher);
+}
+
+/* Trap Ctl-C */
+void trapctlc()
+{
+   signal(SIGINT, sigintcatcher);
+}
 
 static int console_update_history(const char *histfile)
 {
@@ -873,7 +966,8 @@ static int console_init_history(const char *histfile)
    return ret;
 }
 
-bool select_director(const char *director, DIRRES **ret_dir, CONRES **ret_cons)
+static bool select_director(const char *director, const char *console,
+                            DIRRES **ret_dir, CONRES **ret_cons)
 {
    int numcon=0, numdir=0;
    int i=0, item=0;
@@ -952,7 +1046,11 @@ try_again:
    /* Look for a console linked to this director */
    for (i=0; i<numcon; i++) {
       cons = (CONRES *)GetNextRes(R_CONSOLE, (RES *)cons);
-      if (cons->director && strcasecmp(cons->director, dir->hdr.name) == 0) {
+      if (console) {
+         if (strcmp(cons->hdr.name, console) == 0) {
+            break;
+         }
+      } else if (cons->director && strcasecmp(cons->director, dir->hdr.name) == 0) {
          break;
       }
       if (i == (numcon - 1)) {
@@ -960,6 +1058,12 @@ try_again:
       }
    }
 
+   if (cons == NULL && console != NULL) {
+      UnlockRes();
+      senditf(_("Can't find %s in Console list\n"), console);
+      return 0;
+   }
+
    /* Look for the first non-linked console */
    if (cons == NULL) {
       for (i=0; i<numcon; i++) {
@@ -994,7 +1098,8 @@ int main(int argc, char *argv[])
 {
    int ch;
    char *director = NULL;
-   bool list_directors=false;
+   char *console = NULL;
+   bool list_directors=false, list_consoles=false;
    bool no_signals = false;
    bool test_config = false;
    JCR jcr;
@@ -1011,7 +1116,7 @@ int main(int argc, char *argv[])
    working_directory = "/tmp";
    args = get_pool_memory(PM_FNAME);
 
-   while ((ch = getopt(argc, argv, "D:lc:d:nstu:?")) != -1) {
+   while ((ch = getopt(argc, argv, "D:lc:d:nstu:?C:L")) != -1) {
       switch (ch) {
       case 'D':                    /* Director */
          if (director) {
@@ -1020,6 +1125,18 @@ int main(int argc, char *argv[])
          director = bstrdup(optarg);
          break;
 
+      case 'C':                    /* Console */
+         if (console) {
+            free(console);
+         }
+         console = bstrdup(optarg);
+         break;
+
+      case 'L':                    /* Console */
+         list_consoles = true;
+         test_config = true;
+         break;
+
       case 'l':
          list_directors = true;
          test_config = true;
@@ -1094,7 +1211,7 @@ int main(int argc, char *argv[])
       configfile = bstrdup(CONFIG_FILE);
    }
 
-   config = new_config_parser();
+   config = New(CONFIG());
    parse_cons_config(config, configfile, M_ERROR_TERM);
 
    if (init_crypto() != 0) {
@@ -1117,6 +1234,14 @@ int main(int argc, char *argv[])
       UnlockRes();
    }
 
+   if (list_consoles) {
+      LockRes();
+      foreach_res(cons, R_CONSOLE) {
+         senditf("%s\n", cons->hdr.name);
+      }
+      UnlockRes();
+   }
+
    if (test_config) {
       terminate_console(0);
       exit(0);
@@ -1128,7 +1253,7 @@ int main(int argc, char *argv[])
 
    start_watchdog();                        /* Start socket watchdog */
 
-   if (!select_director(director, &dir, &cons)) {
+   if (!select_director(director, console, &dir, &cons)) {
       terminate_console(0);
       return 1;
    }
@@ -1189,13 +1314,15 @@ int main(int argc, char *argv[])
    }
    if (!UA_sock->connect(NULL, 5, 15, heart_beat, "Director daemon", dir->address,
                           NULL, dir->DIRport, 0)) {
+      UA_sock->destroy();
+      UA_sock = NULL;
       terminate_console(0);
       return 1;
    }
    jcr.dir_bsock = UA_sock;
 
    /* If cons==NULL, default console will be used */
-   if (!authenticate_director(&jcr, dir, cons)) {
+   if (!authenticate_director(UA_sock, dir, cons)) {
       terminate_console(0);
       return 1;
    }
@@ -1250,14 +1377,20 @@ static void terminate_console(int sig)
    }
    already_here = true;
    stop_watchdog();
-   config->free_resources();
-   free(config);
+   delete(config);
    config = NULL;
    cleanup_crypto();
+   free(res_head);
+   res_head = NULL;
    free_pool_memory(args);
+#if defined(HAVE_CONIO)
    if (!no_conio) {
       con_term();
    }
+#elif defined(HAVE_READLINE)
+   rl_cleanup_after_signal();
+#else /* !HAVE_CONIO && !HAVE_READLINE */
+#endif
    (void)WSACleanup();               /* Cleanup Windows sockets */
    lmgr_cleanup_main();
 
@@ -1370,11 +1503,21 @@ static int inputcmd(FILE *input, BSOCK *UA_sock)
    return 1;
 }
 
+/* @tall <output-filename> */
+/* Send input/output to both terminal and specified file */
+static int teeallcmd(FILE *input, BSOCK *UA_sock)
+{
+   teeout = true;
+   teein = true;
+   return do_outputcmd(input, UA_sock);
+}
+
 /* @tee <output-filename> */
 /* Send output to both terminal and specified file */
 static int teecmd(FILE *input, BSOCK *UA_sock)
 {
    teeout = true;
+   teein = false;
    return do_outputcmd(input, UA_sock);
 }
 
@@ -1383,6 +1526,7 @@ static int teecmd(FILE *input, BSOCK *UA_sock)
 static int outputcmd(FILE *input, BSOCK *UA_sock)
 {
    teeout = false;
+   teein  = false;
    return do_outputcmd(input, UA_sock);
 }
 
@@ -1401,6 +1545,7 @@ static int do_outputcmd(FILE *input, BSOCK *UA_sock)
          fclose(output);
          output = stdout;
          teeout = false;
+         teein = false;
       }
       return 1;
    }
@@ -1427,15 +1572,30 @@ static int execcmd(FILE *input, BSOCK *UA_sock)
    char line[5000];
    int stat;
    int wait = 0;
+   char *cmd;
 
    if (argc > 3) {
       sendit(_("Too many arguments. Enclose command in double quotes.\n"));
       return 1;
    }
+
+   /* old syntax */
    if (argc == 3) {
       wait = atoi(argk[2]);
    }
-   bpipe = open_bpipe(argk[1], wait, "r");
+   cmd = argk[1];
+
+   /* handle cmd=XXXX and wait=XXXX */
+   for (int i=1; i<argc; i++) {
+      if (strcmp(argk[i], "cmd") == 0) {
+         cmd = argv[i];
+      }
+      if (strcmp(argk[i], "wait") == 0) {
+         wait = atoi(argv[i]);
+      }
+   }
+
+   bpipe = open_bpipe(cmd, wait, "r");
    if (!bpipe) {
       berrno be;
       senditf(_("Cannot popen(\"%s\", \"r\"): ERR=%s\n"),
@@ -1450,12 +1610,11 @@ static int execcmd(FILE *input, BSOCK *UA_sock)
    if (stat != 0) {
       berrno be;
       be.set_errno(stat);
-     senditf(_("Autochanger error: ERR=%s\n"), be.bstrerror());
+      senditf(_("@exec error: ERR=%s\n"), be.bstrerror());
    }
    return 1;
 }
 
-
 /* @echo xxx yyy */
 static int echocmd(FILE *input, BSOCK *UA_sock)
 {
@@ -1492,6 +1651,59 @@ static int sleepcmd(FILE *input, BSOCK *UA_sock)
    return 1;
 }
 
+/* @putfile key /path/to/file
+ *
+ * The Key parameter is needed to use the file on the director side.
+ */
+static int putfilecmd(FILE *input, BSOCK *UA_sock)
+{
+   int i = 0;
+   const char *key = "putfile";
+   const char *fname;
+   FILE *fp;
+
+   if (argc != 3) {
+      sendit("Usage: @putfile key file\n");
+      return 1;
+   }
+
+   key = argk[1];
+   fname = argk[2];
+
+   if (!key || !fname) {
+      senditf("Syntax error in @putfile command\n");
+      return 1;
+   }
+
+   fp = fopen(fname, "r");
+   if (!fp) {
+      berrno be;
+      senditf("Unable to open %s. ERR=%s\n", fname, be.bstrerror(errno));
+      return 1;
+   }
+
+   UA_sock->fsend(".putfile key=\"%s\"", key);
+
+   /* Just read the file and send it to the director */
+   while (!feof(fp)) {
+      i = fread(UA_sock->msg, 1, sizeof_pool_memory(UA_sock->msg) - 1, fp);
+      if (i > 0) {
+         UA_sock->msg[i] = 0;
+         UA_sock->msglen = i;
+         UA_sock->send();
+      }
+   }
+
+   UA_sock->signal(BNET_EOD);
+   fclose(fp);
+
+   /* Get the file name associated */
+   while (UA_sock->recv() > 0) {
+      senditf("%s", UA_sock->msg);
+   }
+   return 1;
+}
+
 /* @time */
 static int timecmd(FILE *input, BSOCK *UA_sock)
 {
