@@ -1,38 +1,28 @@
+/*
+   Bacula(R) - The Network Backup Solution
+
+   Copyright (C) 2000-2017 Kern Sibbald
+
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
+
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
+
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
+*/
 /*
  *
  *   Bacula Director -- User Agent Database Query Commands
  *
  *     Kern Sibbald, December MMI
  *
- *   Version $Id$
  */
-/*
-   Bacula® - The Network Backup Solution
-
-   Copyright (C) 2001-2006 Free Software Foundation Europe e.V.
-
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
-
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
-*/
 
 #include "bacula.h"
 #include "dird.h"
@@ -55,20 +45,23 @@ static POOLMEM *substitute_prompts(UAContext *ua,
  *   SQL statement possibly terminated by ;
  *   :next query prompt
  */
-int querycmd(UAContext *ua, const char *cmd)
+int query_cmd(UAContext *ua, const char *cmd)
 {
    FILE *fd = NULL;
    POOLMEM *query = get_pool_memory(PM_MESSAGE);
    char line[1000];
-   int i, item, len;
+   int i, len;
+   int item = 0;
+   long val;
    char *prompt[9];
    int nprompt = 0;;
    char *query_file = director->query_file;
+   bool must_prompt = true;
 
    if (!open_client_db(ua)) {
       goto bail_out;
    }
-   if ((fd=fopen(query_file, "rb")) == NULL) {
+   if ((fd=bfopen(query_file, "rb")) == NULL) {
       berrno be;
       ua->error_msg(_("Could not open %s: ERR=%s\n"), query_file,
          be.bstrerror());
@@ -82,7 +75,21 @@ int querycmd(UAContext *ua, const char *cmd)
          add_prompt(ua, line+1);
       }
    }
-   if ((item=do_prompt(ua, "", _("Choose a query"), NULL, 0)) < 0) {
+   if (ua->argc >= 2) {
+       errno = 0;
+       val = strtol(ua->argk[1], NULL, 10) - 1;
+       if (val < 0) {
+          errno = 1;
+       }
+       if (errno != 0) {
+          ua->error_msg(_("Invalid command line query item specified.\n"));
+          must_prompt = true;
+       } else {
+          item = val;
+          must_prompt = false;
+      }
+   }
+   if (must_prompt && (item=do_prompt(ua, "", _("Choose a query"), NULL, 0)) < 0) {
       goto bail_out;
    }
    rewind(fd);
@@ -251,13 +258,13 @@ static POOLMEM *substitute_prompts(UAContext *ua,
 /*
  * Get general SQL query for Catalog
  */
-int sqlquerycmd(UAContext *ua, const char *cmd)
+int sqlquery_cmd(UAContext *ua, const char *cmd)
 {
    POOL_MEM query(PM_MESSAGE);
    int len;
    const char *msg;
 
-   if (!open_client_db(ua)) {
+   if (!open_new_client_db(ua)) {
       return 1;
    }
    *query.c_str() = 0;
