@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2004-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2018 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*                               -*- Mode: C -*-
  * compat.h --
@@ -36,6 +27,8 @@
  * Created On      : Fri Jan 30 13:00:51 2004
  */
 
+#include <winnt.h>
+#include <ws2ipdef.h>
 
 #if !defined(__COMPAT_H_)
 #define __COMPAT_H_
@@ -56,36 +49,35 @@
 
 #include <malloc.h>
 
+#ifdef HAVE_MINGW_W64
+/* Was defined in pthread.h before */
+# define localtime_r( _clock, _result ) \
+        ( *(_result) = *localtime( (_clock) ), \
+          (_result) )
+
+# define strtok_r( _s, _sep, _lasts ) \
+        ( *(_lasts) = strtok( (_s), (_sep) ) )
+
+# define asctime_r( _tm, _buf ) \
+        ( strcpy( (_buf), asctime( (_tm) ) ), \
+          (_buf) )
+
+# define ctime_r( _clock, _buf ) \
+        ( strcpy( (_buf), ctime( (_clock) ) ),  \
+          (_buf) )
+
+# define gmtime_r( _clock, _result ) \
+        ( *(_result) = *gmtime( (_clock) ), \
+          (_result) )
+
+#define rand_r( _seed ) \
+        ( _seed == _seed? rand() : rand() )
+
+#endif
 #ifdef MINGW64
 #include <direct.h>
+#include <ntdef.h>
 #define _declspec __declspec
-
-/* Missing in 64 bit mingw */
-typedef struct _REPARSE_DATA_BUFFER {
-        DWORD  ReparseTag;
-        WORD   ReparseDataLength;
-        WORD   Reserved;
-        union {
-                struct {
-                        WORD   SubstituteNameOffset;
-                        WORD   SubstituteNameLength;
-                        WORD   PrintNameOffset;
-                        WORD   PrintNameLength;
-                        WCHAR PathBuffer[1];
-                } SymbolicLinkReparseBuffer;
-                struct {
-                        WORD   SubstituteNameOffset;
-                        WORD   SubstituteNameLength;
-                        WORD   PrintNameOffset;
-                        WORD   PrintNameLength;
-                        WCHAR PathBuffer[1];
-                } MountPointReparseBuffer;
-                struct {
-                        BYTE   DataBuffer[1];
-                } GenericReparseBuffer;
-        } DUMMYUNIONNAME;
-} REPARSE_DATA_BUFFER, *PREPARSE_DATA_BUFFER;
-
 #endif
 
 #include <winioctl.h>
@@ -200,6 +192,9 @@ int gettimeofday(struct timeval *, struct timezone *);
 #define ENODATA 61
 #endif
 
+/*
+ * Stat packet that we use for Windows
+ */
 struct stat
 {
     _dev_t      st_dev;
@@ -215,8 +210,13 @@ struct stat
     time_t      st_ctime;
     uint32_t    st_blksize;
     uint64_t    st_blocks;
+    uint32_t    st_fattrs;  /* Windows file attributes */
 };
 
+#ifndef SOCK_CLOEXEC
+#define SOCK_CLOEXEC 0x00000000
+#endif
+
 #undef  S_IFMT
 #define S_IFMT         0170000         /* file type mask */
 #undef  S_IFDIR
@@ -270,8 +270,10 @@ struct stat
 
 typedef  BOOL (*t_pVSSPathConvert)(const char *szFilePath, char *szShadowPath, int nBuflen);
 typedef  BOOL (*t_pVSSPathConvertW)(const wchar_t  *szFilePath, wchar_t  *szShadowPath, int nBuflen);
+/* To know if we can use the VSSPathConvert function */
+typedef  BOOL (*t_pVSSPathConverter)();
 
-void SetVSSPathConvert(t_pVSSPathConvert pPathConvert, t_pVSSPathConvertW pPathConvertW);
+void SetVSSPathConvert(t_pVSSPathConverter pPathConverter, t_pVSSPathConvert pPathConvert, t_pVSSPathConvertW pPathConvertW);
 
 int lchown(const char *, uid_t uid, gid_t gid);
 int chown(const char *, uid_t uid, gid_t gid);
@@ -282,16 +284,6 @@ int chmod(const char *, mode_t mode);
 #define F_GETFL      3
 #define F_SETFL      4
 
-int win32_tape_open(const char *file, int flags, ...);
-int win32_tape_ioctl(int fd, unsigned long int request, ...);
-int win32_tape_close(int fd);
-ssize_t win32_tape_read(int fd, void *buffer, size_t count);
-ssize_t win32_tape_write(int fd, const void *buffer, size_t count);
-
-ssize_t win32_read(int fd, void *buffer, size_t count);
-ssize_t win32_write(int fd, const void *buffer, size_t count);
-int win32_ioctl(int fd, unsigned long int req, ...);
-
 #ifndef MINGW64
 #define open   _open
 #endif
@@ -300,7 +292,8 @@ int fcntl(int fd, int cmd, long arg);
 int fstat(intptr_t fd, struct stat *sb);
 
 int inet_aton(const char *cp, struct in_addr *inp);
-int kill(int pid, int signo);
+int binet_pton(int af, const char *src, void *dst);
+int kill(pid_t pid, int signo);
 int pipe(int []);
 int fork();
 int waitpid(int, int *, int);
@@ -333,6 +326,7 @@ int lstat(const char *, struct stat *);
 int stat(const char *file, struct stat *sb);
 long pathconf(const char *, int);
 int readlink(const char *, char *, int);
+int symlink(const char *path1, const char *path2);
 #define _PC_PATH_MAX 1
 #define _PC_NAME_MAX 2
 
@@ -352,18 +346,20 @@ struct group {
 struct passwd *getpwuid(uid_t);
 struct group *getgrgid(uid_t);
 
+#ifdef xxx_needed
 struct sigaction {
     int sa_flags;
     void (*sa_handler)(int);
 };
 #define sigfillset(x)
 #define sigaction(a, b, c)
+#endif
 
 #define mkdir(p, m) win32_mkdir(p)
 #define unlink win32_unlink
 #define chdir win32_chdir
 #define chmod win32_chmod
-extern "C" void syslog(int type, const char *fmt, ...);
+extern void syslog(int type, const char *fmt, ...);
 #if !defined(LOG_DAEMON)
 #define LOG_DAEMON 0
 #endif
@@ -409,10 +405,6 @@ void closelog();
 void openlog(const char *ident, int option, int facility);
 #endif //HAVE_MINGW
 
-/* Don't let OS go to sleep (usually a Laptop) while we are backing up */
-void prevent_os_suspensions();
-void allow_os_suspensions();
-
 typedef DWORD EXECUTION_STATE;
 #ifndef ES_CONTINUOUS
 #define ES_CONTINUOUS            0x80000000
@@ -436,11 +428,19 @@ extern void LogErrorMsg(const char *message);
 #define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
 #endif
 
+/*
+ * Since the FileAttributes flag in Windows does not hold the
+ *  dedup bit, we simplify the code by defining our own
+ *  bit.
+ */
+#define FILE_ATTRIBUTE_DEDUP 0x80000000
+#define CREATE_FOR_EXPORT 0 /* default in OpenEncryptedFileRaw */
+
 #if defined(_MSC_VER)
 inline unsigned long ffs(unsigned long word)
 {
    unsigned long  index;
-   
+
    if (_BitScanForward(&index, word) != 0)
       return index + 1;
    else
@@ -453,8 +453,17 @@ inline unsigned long ffs(unsigned long word)
 
 
 int win32_ftruncate(int fd, int64_t length);
+int mkstemp(char *t);
+
+void malloc_trim(int);
+uint64_t get_memory_info(char *buf, int buflen);
 
 #undef ftruncate
 #define ftruncate win32_ftruncate
 
+#ifndef O_DIRECTORY
+#define O_DIRECTORY 0100000000
+#endif
+#define O_ENCRYPTED 0200000000
+
 #endif /* __COMPAT_H_ */
