@@ -1,3 +1,43 @@
+/*
+ *
+ * Note: 25 May 2018. This code appears not to have a copyright, however,
+ *   it appears to come from exactly the same code source as bsnprintf.c
+ *   in the Bacula src/lib.  The code in that file has two licenses that
+ * am adding here:
+ */
+
+/*
+   Bacula(R) - The Network Backup Solution
+
+   Copyright (C) 2000-2018 Kern Sibbald
+
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
+
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
+
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
+*/
+/*
+ * Copyright Patrick Powell 1995
+ *
+ * This code is based on code written by Patrick Powell
+ * (papowell@astart.com) It may be used for any purpose as long
+ * as this notice remains intact on all source code distributions.
+ *
+ * Adapted for Bacula -- note there were lots of bugs in
+ *     the original code: %lld and %s were seriously broken, and
+ *     with FP turned off %f seg faulted.
+ *
+ *   Kern Sibbald, November MMV
+ *
+ */
 /**************************************************************
  * Original:
  * Patrick Powell Tue Apr 11 09:48:21 PDT 1995
@@ -28,9 +68,9 @@
  *    original.  Also, there is now a builtin-test, just compile with:
  *           gcc -DTEST_SNPRINTF -o snprintf snprintf.c -lm
  *    and run snprintf for results.
- * 
+ *
  *  Thomas Roessler <roessler@guug.de> 01/27/98 for mutt 0.89i
- *    The PGP code was using unsigned hexadecimal formats. 
+ *    The PGP code was using unsigned hexadecimal formats.
  *    Unfortunately, unsigned formats simply didn't work.
  *
  *  Michael Elkins <me@cs.hmc.edu> 03/05/98 for mutt 0.90.8
@@ -44,7 +84,7 @@
  *    right.
  *
  *  Ben Lindstrom <mouring@eviladmin.org> 02/12/01 for OpenSSH
- *    Removed all hint of VARARGS stuff and banished it to the void, 
+ *    Removed all hint of VARARGS stuff and banished it to the void,
  *    and did a bit of KNF style work to make things a bit more
  *    acceptable.  Consider stealing from mutt or enlightenment.
  **************************************************************/
@@ -54,20 +94,20 @@
 
 typedef void (prfun)(char *, size_t *, size_t, int);
 
-int 
+int
 dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun);
 
 
-static void 
-fmtstr(char *buffer, size_t *currlen, size_t maxlen, char *value, int flags, 
+static void
+fmtstr(char *buffer, size_t *currlen, size_t maxlen, char *value, int flags,
        int min, int max, prfun);
 
-static void 
-fmtint(char *buffer, size_t *currlen, size_t maxlen, INT64 value, int base, 
+static void
+fmtint(char *buffer, size_t *currlen, size_t maxlen, INT64 value, int base,
        int min, int max, int flags, prfun);
 
-static void 
-fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue, 
+static void
+fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
       int min, int max, int flags, prfun);
 
 static void
@@ -124,20 +164,20 @@ dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun outch)
     int flags = 0;
     int cflags = 0;
     size_t currlen = 0;
-  
+
     ch = *format++;
 
     while (state != DP_S_DONE)
     {
-        if ((ch == '\0') || (currlen >= maxlen)) 
+        if ((ch == '\0') || (currlen >= maxlen))
             state = DP_S_DONE;
 
         switch (state)
         {
         case DP_S_DEFAULT:
-            if (ch == '%') 
+            if (ch == '%')
                 state = DP_S_FLAGS;
-            else 
+            else
                 outch(buffer, &currlen, maxlen, ch);
             ch = *format++;
             break;
@@ -181,7 +221,7 @@ dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun outch)
                 ch = *format++;
                 state = DP_S_DOT;
             }
-            else 
+            else
                 state = DP_S_DOT;
             break;
         case DP_S_DOT:
@@ -190,7 +230,7 @@ dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun outch)
                 state = DP_S_MAX;
                 ch = *format++;
             }
-            else 
+            else
                 state = DP_S_MOD;
             break;
         case DP_S_MAX:
@@ -207,7 +247,7 @@ dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun outch)
                 ch = *format++;
                 state = DP_S_MOD;
             }
-            else 
+            else
                 state = DP_S_MOD;
             break;
         case DP_S_MOD:
@@ -244,7 +284,7 @@ dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun outch)
             {
             case 'b':
                 flags |= DP_F_UNSIGNED;
-                if (cflags == DP_C_SHORT) 
+                if (cflags == DP_C_SHORT)
                     value = va_arg(args, unsigned int);
                 else if (cflags == DP_C_LONG)
                     value = va_arg(args, unsigned long int);
@@ -256,7 +296,7 @@ dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun outch)
                 break;
             case 'd':
             case 'i':
-                if (cflags == DP_C_SHORT) 
+                if (cflags == DP_C_SHORT)
                     value = va_arg(args, int);
                 else if (cflags == DP_C_LONG)
                     value = va_arg(args, long int);
@@ -401,35 +441,32 @@ dopr(char *buffer, size_t maxlen, const char *format, va_list args, prfun outch)
 }
 
 static void
-fmtstr(char *buffer, size_t *currlen, size_t maxlen, 
+fmtstr(char *buffer, size_t *currlen, size_t maxlen,
        char *value, int flags, int min, int max, prfun outch)
 {
     int padlen, strln;     /* amount to pad */
     int cnt = 0;
-  
-    if (value == NULL) 
+
+    if (value == NULL)
         value = (char *)"<NULL>";
 
     for (strln = 0; value[strln]; ++strln); /* strlen */
     padlen = min - strln;
-    if (padlen < 0) 
+    if (padlen < 0)
         padlen = 0;
-    if (flags & DP_F_MINUS) 
+    if (flags & DP_F_MINUS)
         padlen = -padlen; /* Left Justify */
 
-    while ((padlen > 0) && (cnt < max))
-    {
+    while ((padlen > 0) && (cnt < max)) {
         outch(buffer, currlen, maxlen, ' ');
         --padlen;
         ++cnt;
     }
-    while (*value && (cnt < max))
-    {
+    while (*value && (cnt < max)) {
         outch(buffer, currlen, maxlen, *value++);
         ++cnt;
     }
-    while ((padlen < 0) && (cnt < max))
-    {
+    while ((padlen < 0) && (cnt < max)) {
         outch(buffer, currlen, maxlen, ' ');
         ++padlen;
         ++cnt;
@@ -438,8 +475,8 @@ fmtstr(char *buffer, size_t *currlen, size_t maxlen,
 
 /* Have to handle DP_F_NUM(ie 0x and 0 alternates) */
 
-static void 
-fmtint(char *buffer, size_t *currlen, size_t maxlen, 
+static void
+fmtint(char *buffer, size_t *currlen, size_t maxlen,
        INT64 value, int base, int min, int max, int flags, prfun outch)
 {
     UINT64 uvalue;
@@ -449,16 +486,14 @@ fmtint(char *buffer, size_t *currlen, size_t maxlen,
     int spadlen = 0; /* amount to space pad */
     int zpadlen = 0; /* amount to zero pad */
     int caps = 0;
-    
+
     if (max < 0)
         max = 0;
-    
+
     uvalue = value;
 
-    if (!(flags & DP_F_UNSIGNED))
-    {
-        if (value < 0)
-        {
+    if (!(flags & DP_F_UNSIGNED)) {
+        if (value < 0) {
             signvalue = '-';
             uvalue = -value;
         }
@@ -467,17 +502,16 @@ fmtint(char *buffer, size_t *currlen, size_t maxlen,
         else if (flags & DP_F_SPACE)
             signvalue = ' ';
     }
-  
-    if (flags & DP_F_UP) 
+
+    if (flags & DP_F_UP)
         caps = 16; /* Should characters be upper case? */
 
-    do
-    {
+    do {
         convert[place++] = digitval[(uvalue%base)+caps];
         uvalue = (uvalue / (unsigned)base);
     } while (uvalue && (place < 20));
 
-    if (place == 20) 
+    if (place == 20)
         place--;
 
     convert[place] = 0;
@@ -489,49 +523,44 @@ fmtint(char *buffer, size_t *currlen, size_t maxlen,
         zpadlen = 0;
     if (spadlen < 0)
         spadlen = 0;
-    if (flags & DP_F_ZERO)
-    {
+    if (flags & DP_F_ZERO) {
         zpadlen = MAX(zpadlen, spadlen);
         spadlen = 0;
     }
-    if (flags & DP_F_MINUS) 
+    if (flags & DP_F_MINUS)
         spadlen = -spadlen; /* Left Justifty */
 
 
     /* Spaces */
-    while (spadlen > 0)
-    {
+    while (spadlen > 0) {
         outch(buffer, currlen, maxlen, ' ');
         --spadlen;
     }
 
     /* Sign */
-    if (signvalue) 
+    if (signvalue)
         outch(buffer, currlen, maxlen, signvalue);
 
     /* Zeros */
-    if (zpadlen > 0)
-    {
-        while (zpadlen > 0)
-        {
+    if (zpadlen > 0) {
+        while (zpadlen > 0) {
             outch(buffer, currlen, maxlen, '0');
             --zpadlen;
         }
     }
 
     /* Digits */
-    while (place > 0) 
+    while (place > 0)
         outch(buffer, currlen, maxlen, convert[--place]);
-  
+
     /* Left Justified spaces */
-    while (spadlen < 0)
-    {
+    while (spadlen < 0) {
         outch(buffer, currlen, maxlen, ' ');
         ++spadlen;
     }
 }
 
-static long double 
+static long double
 pow10(int exp)
 {
     long double result = 1;
@@ -541,11 +570,11 @@ pow10(int exp)
         result *= 10;
         exp--;
     }
-  
+
     return result;
 }
 
-static long 
+static long
 round(long double value)
 {
     long intpart = (long)value;
@@ -557,8 +586,8 @@ round(long double value)
     return intpart;
 }
 
-static void 
-fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue, 
+static void
+fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
       int min, int max, int flags, prfun outch)
 {
     char iconvert[20];
@@ -567,12 +596,12 @@ fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
     int iplace = 0;
     int fplace = 0;
     int padlen = 0; /* amount to pad */
-    int zpadlen = 0; 
+    int zpadlen = 0;
     long intpart;
     long fracpart;
     long double ufvalue;
-  
-    /* 
+
+    /*
      * AIX manpage says the default is 0, but Solaris says the default
      * is 6, and sprintf on AIX defaults to 6
      */
@@ -590,8 +619,8 @@ fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
 
     intpart = (long)ufvalue;
 
-    /* 
-     * Sorry, we only support 9 digits past the decimal because of our 
+    /*
+     * Sorry, we only support 9 digits past the decimal because of our
      * conversion method
      */
     if (max > 9)
@@ -615,7 +644,7 @@ fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
         intpart = (intpart / 10);
     } while (intpart && (iplace < 20));
 
-    if (iplace == 20) 
+    if (iplace == 20)
         iplace--;
 
     iconvert[iplace] = 0;
@@ -627,19 +656,19 @@ fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
         fracpart = (fracpart / 10);
     } while (fracpart && (fplace < 20));
 
-    if (fplace == 20) 
+    if (fplace == 20)
         fplace--;
 
     fconvert[fplace] = 0;
 
     /* -1 for decimal point, another -1 if we are printing a sign */
-    padlen = min - iplace - max - 1 - ((signvalue) ? 1 : 0); 
+    padlen = min - iplace - max - 1 - ((signvalue) ? 1 : 0);
     zpadlen = max - fplace;
     if (zpadlen < 0)
         zpadlen = 0;
-    if (padlen < 0) 
+    if (padlen < 0)
         padlen = 0;
-    if (flags & DP_F_MINUS) 
+    if (flags & DP_F_MINUS)
         padlen = -padlen; /* Left Justifty */
 
     if ((flags & DP_F_ZERO) && (padlen > 0))
@@ -663,10 +692,10 @@ fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
         --padlen;
     }
 
-    if (signvalue) 
+    if (signvalue)
         outch(buffer, currlen, maxlen, signvalue);
 
-    while (iplace > 0) 
+    while (iplace > 0)
         outch(buffer, currlen, maxlen, iconvert[--iplace]);
 
     /*
@@ -675,7 +704,7 @@ fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
      */
     outch(buffer, currlen, maxlen, '.');
 
-    while (fplace > 0) 
+    while (fplace > 0)
         outch(buffer, currlen, maxlen, fconvert[--fplace]);
 
     while (zpadlen > 0)
@@ -691,14 +720,14 @@ fmtfp(char *buffer, size_t *currlen, size_t maxlen, long double fvalue,
     }
 }
 
-static void 
+static void
 dopr_outch(char *buffer, size_t *currlen, size_t maxlen, int c)
 {
     if (c == -1)
     {
-        if (*currlen < maxlen - 1) 
+        if (*currlen < maxlen - 1)
             buffer[*currlen] = '\0';
-        else 
+        else
             buffer[maxlen - 1] = '\0';
     }
     else if (*currlen < maxlen)
@@ -706,7 +735,7 @@ dopr_outch(char *buffer, size_t *currlen, size_t maxlen, int c)
 }
 
 int
-__sprintf(char *str, const char *fmt, ...) 
+__sprintf(char *str, const char *fmt, ...)
 {
     int rval;
     va_list ap;
@@ -717,7 +746,7 @@ __sprintf(char *str, const char *fmt, ...)
 }
 
 
-int 
+int
 __snprintf(char *str, size_t count, const char *fmt, ...)
 {
     int rval;
