@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2005-2010 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2018 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 //                              -*- Mode: C++ -*-
 // vss.cpp -- Interface to Volume Shadow Copies (VSS)
@@ -38,7 +29,7 @@
 #ifdef WIN32_VSS
 
 #include "bacula.h"
-#include "jcr.h"
+#include "filed/filed.h"
 
 #undef setlocale
 
@@ -55,13 +46,15 @@ using namespace std;
 
 /* 
  * Kludges to get Vista code to compile.             
- *  KES - June 2007
+ *  by Kern Sibbald - June 2007
  */
 #define __in  IN
 #define __out OUT
 #define __RPC_unique_pointer
 #define __RPC_string
+#ifndef __RPC__out_ecount_part
 #define __RPC__out_ecount_part(x, y)
+#endif
 #define __RPC__deref_inout_opt
 #define __RPC__out
 
@@ -74,9 +67,6 @@ using namespace std;
 #include <strsafe.h>
 #endif
 
-BOOL VSSPathConvert(const char *szFilePath, char *szShadowPath, int nBuflen);
-BOOL VSSPathConvertW(const wchar_t *szFilePath, wchar_t *szShadowPath, int nBuflen);
-
 #ifdef HAVE_MINGW
 class IXMLDOMDocument;
 #endif
@@ -106,13 +96,6 @@ class IXMLDOMDocument;
    #include "inc/Win2003/vswriter.h"
    #include "inc/Win2003/vsbackup.h"
 #endif
-   
-/* In VSSAPI.DLL */
-typedef HRESULT (STDAPICALLTYPE* t_CreateVssBackupComponents)(OUT IVssBackupComponents **);
-typedef void (APIENTRY* t_VssFreeSnapshotProperties)(IN VSS_SNAPSHOT_PROP*);
-   
-static t_CreateVssBackupComponents p_CreateVssBackupComponents = NULL;
-static t_VssFreeSnapshotProperties p_VssFreeSnapshotProperties = NULL;
 
 #include "vss.h"
 
@@ -230,7 +213,6 @@ static void JmsgVssWriterStatus(JCR *jcr, int msg_status, VSS_WRITER_STATE eWrit
    Jmsg(jcr, msg_status, 0, "VSS Writer \"%s\" has invalid state. ERR=%s\n", writer_name, errmsg);
 }
 
-
 /*  
  *
  * some helper functions 
@@ -238,50 +220,12 @@ static void JmsgVssWriterStatus(JCR *jcr, int msg_status, VSS_WRITER_STATE eWrit
  *
  */
 
+
+// Defined in vss.cpp
 // Append a backslash to the current string 
-inline wstring AppendBackslash(wstring str)
-{
-    if (str.length() == 0) {
-        return wstring(L"\\");
-    }
-    if (str[str.length() - 1] == L'\\') {
-        return str;
-    }
-    return str.append(L"\\");
-}
-
+wstring AppendBackslash(wstring str);
 // Get the unique volume name for the given path
-inline wstring GetUniqueVolumeNameForPath(wstring path)
-{
-    if (path.length() <= 0) {
-       return L"";
-    }
-
-    // Add the backslash termination, if needed
-    path = AppendBackslash(path);
-
-    // Get the root path of the volume
-    wchar_t volumeRootPath[MAX_PATH];
-    wchar_t volumeName[MAX_PATH];
-    wchar_t volumeUniqueName[MAX_PATH];
-
-    if (!p_GetVolumePathNameW || !p_GetVolumePathNameW((LPCWSTR)path.c_str(), volumeRootPath, MAX_PATH)) {
-       return L"";
-    }
-    
-    // Get the volume name alias (might be different from the unique volume name in rare cases)
-    if (!p_GetVolumeNameForVolumeMountPointW || !p_GetVolumeNameForVolumeMountPointW(volumeRootPath, volumeName, MAX_PATH)) {
-       return L"";
-    }
-    
-    // Get the unique volume name    
-    if (!p_GetVolumeNameForVolumeMountPointW(volumeName, volumeUniqueName, MAX_PATH)) {
-       return L"";
-    }
-    
-    return volumeUniqueName;
-}
-
+wstring GetUniqueVolumeNameForPath(wstring path, wstring &rootPath);
 
 // Helper macro for quick treatment of case statements for error codes
 #define GEN_MERGE(A, B) A##B
@@ -316,33 +260,13 @@ inline const wchar_t* GetStringFromWriterStatus(VSS_WRITER_STATE eWriterStatus)
 
 // Constructor
 
-#ifdef HAVE_VSS64
-/* 64 bit entrypoint name */
-#define VSSVBACK_ENTRY "?CreateVssBackupComponents@@YAJPEAPEAVIVssBackupComponents@@@Z"
-#else
-/* 32 bit entrypoint name */
-#define VSSVBACK_ENTRY "?CreateVssBackupComponents@@YGJPAPAVIVssBackupComponents@@@Z"
-#endif
-
 VSSClientGeneric::VSSClientGeneric()
 {
-   m_hLib = LoadLibraryA("VSSAPI.DLL");
-   if (m_hLib) {      
-      p_CreateVssBackupComponents = (t_CreateVssBackupComponents)
-         GetProcAddress(m_hLib, VSSVBACK_ENTRY);
-      p_VssFreeSnapshotProperties = (t_VssFreeSnapshotProperties)
-          GetProcAddress(m_hLib, "VssFreeSnapshotProperties");      
-   } 
 }
 
-
-
 // Destructor
 VSSClientGeneric::~VSSClientGeneric()
 {
-   if (m_hLib) {
-      FreeLibrary(m_hLib);
-   }
 }
 
 // Initialize the COM infrastructure and the internal pointers
@@ -358,43 +282,29 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
       return false;
    }
 
+   if (m_VolumeList) {
+      delete m_VolumeList;
+   }
+
+   m_VolumeList = New(MTab());  // TODO: See if we do this part only in backup
+   if (!m_VolumeList->get()) {
+      Jmsg(m_jcr, M_ERROR, 0, "Unable to list devices and volumes.\n");
+      return false;
+   }
+
    HRESULT hr;
    // Initialize COM
    if (!m_bCoInitializeCalled) {
-      hr = CoInitialize(NULL);
+      hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
       if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: CoInitialize returned 0x%08X\n", hr);
-         JmsgVssApiStatus(m_jcr, M_FATAL, hr, "CoInitialize");
+         Dmsg1(0, "VSSClientGeneric::Initialize: CoInitializeEx returned 0x%08X\n", hr);
+         JmsgVssApiStatus(m_jcr, M_FATAL, hr, "CoInitializeEx");
          errno = b_errno_win32;
          return false;
       }
       m_bCoInitializeCalled = true;
    }
 
-   // Initialize COM security
-   if (!m_bCoInitializeSecurityCalled) {
-      hr =
-         CoInitializeSecurity(
-         NULL,                           //  Allow *all* VSS writers to communicate back!
-         -1,                             //  Default COM authentication service
-         NULL,                           //  Default COM authorization service
-         NULL,                           //  reserved parameter
-         RPC_C_AUTHN_LEVEL_PKT_PRIVACY,  //  Strongest COM authentication level
-         RPC_C_IMP_LEVEL_IDENTIFY,       //  Minimal impersonation abilities 
-         NULL,                           //  Default COM authentication settings
-         EOAC_NONE,                      //  No special options
-         NULL                            //  Reserved parameter
-         );
-
-      if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: CoInitializeSecurity returned 0x%08X\n", hr);
-         JmsgVssApiStatus(m_jcr, M_FATAL, hr, "CoInitializeSecurity");
-         errno = b_errno_win32;
-         return false;
-      }
-      m_bCoInitializeSecurityCalled = true;
-   }
-
    // Release the any old IVssBackupComponents interface 
    if (pVssObj) {
       pVssObj->Release();
@@ -499,7 +409,7 @@ bool VSSClientGeneric::WaitAndCheckForAsyncOperation(IVssAsync* pAsync)
    // Check the result of the asynchronous operation
    HRESULT hrReturned = S_OK;
 
-   int timeout = 600; // 10 minutes....
+   int timeout = 1800; // 30 minutes ...
 
    int queryErrors = 0;
    do {
@@ -517,22 +427,44 @@ bool VSSClientGeneric::WaitAndCheckForAsyncOperation(IVssAsync* pAsync)
       return true;
    }
    
-   JmsgVssApiStatus(m_jcr, M_FATAL, hr, "Query Async Status after 10 minute wait");
+   JmsgVssApiStatus(m_jcr, M_FATAL, hr, "Query Async Status after 30 minute wait");
    return false;
 }
 
-bool VSSClientGeneric::CreateSnapshots(char* szDriveLetters)
+static int volume_cmp(void *e1, void *e2)
+{
+   WCHAR *v1 = (WCHAR *) e1;
+   MTabEntry *v2 = (MTabEntry *) e2;
+   return wcscmp(v1, v2->volumeName);
+}
+
+static pthread_mutex_t create_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+bool VSSClientGeneric::CreateSnapshots(alist *mount_points)
 {
    IVssBackupComponents *pVssObj;
+   bool ret = false;
    HRESULT hr;
 
+   /* AddToSnapshotSet */
+   CComPtr<IVssAsync>  pAsync1;
+   CComPtr<IVssAsync>  pAsync2;   
+   VSS_ID pid;
+
+   /* While testing the concurrent snapshot creation, I found out that the entire snapshot
+    * creation process should be protected by a mutex. (InitializeForBackups and CreateSnapshots).
+    */
+
+   /* Create only one snapshot set at a time */
+   P(create_mutex);
+
    /* szDriveLetters contains all drive letters in uppercase */
    /* if a drive can not being added, it's converted to lowercase in szDriveLetters */
    /* http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vss/base/ivssbackupcomponents_startsnapshotset.asp */
    if (!m_pVssObject || m_bBackupIsInitialized) {
       Jmsg(m_jcr, M_FATAL, 0, "No pointer to VssObject or Backup is not Initialized\n");
       errno = ENOSYS;
-      return false;  
+      goto bail_out;
    }
 
    m_uidCurrentSnapshotSet = GUID_NULL;
@@ -544,30 +476,23 @@ bool VSSClientGeneric::CreateSnapshots(char* szDriveLetters)
    if (FAILED(hr)) {
       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "StartSnapshotSet");
       errno = ENOSYS;
-      return false;
+      goto bail_out;
    }
-      
 
-   /* AddToSnapshotSet */
-
-   wchar_t szDrive[3];
-   szDrive[1] = ':';
-   szDrive[2] = 0;
-
-   wstring volume;
-
-   CComPtr<IVssAsync>  pAsync1;
-   CComPtr<IVssAsync>  pAsync2;   
-   VSS_ID pid;
-
-   for (size_t i=0; i < strlen (szDriveLetters); i++) {
-      szDrive[0] = szDriveLetters[i];
-      volume = GetUniqueVolumeNameForPath(szDrive);
+   /*
+    * Now try all paths in case they are mount points
+    */
+   for (int i=0; i < mount_points->size(); i++) {
+      wchar_t *p = (wchar_t *)mount_points->get(i);
       // store uniquevolumname
-      if (SUCCEEDED(pVssObj->AddToSnapshotSet((LPWSTR)volume.c_str(), GUID_NULL, &pid))) {
-         wcsncpy (m_wszUniqueVolumeName[szDriveLetters[i]-'A'], (LPWSTR) volume.c_str(), MAX_PATH);
-      } else {            
-         szDriveLetters[i] = tolower(szDriveLetters[i]);               
+      if (SUCCEEDED(pVssObj->AddToSnapshotSet(p, GUID_NULL, &pid))) {
+         MTabEntry *elt = (MTabEntry*)m_VolumeList->entries->search(p, volume_cmp);
+         ASSERT2(elt, "Should find the volume in the list");
+         Jmsg(m_jcr, M_INFO, 0, "    Snapshot mount point: %ls\n", elt->first());
+         Dmsg1(50, "AddToSnapshot OK for Vol: %ls\n", p);
+      } else {
+         //Dmsg1(50, "AddToSnapshot() failed for Vol: %ls\n", (LPWSTR)volume.c_str());
+         //Dmsg1(50, "AddToSnapshot() failed for path: %s\n", p);
       }
    }
 
@@ -576,19 +501,21 @@ bool VSSClientGeneric::CreateSnapshots(char* szDriveLetters)
    if (FAILED(hr)) {
       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "PrepareForBackup");
       errno = b_errno_win32;
-      return false;   
+      goto bail_out;
    }
    
    // Waits for the async operation to finish and checks the result
    if (!WaitAndCheckForAsyncOperation(pAsync1.p)) {
+      /* Error message already printed */
       errno = b_errno_win32;
-      return false;
+      goto bail_out;
    }
 
    /* get latest info about writer status */
    if (!CheckWriterStatus()) {
+      /* Error message already printed */
       errno = b_errno_win32;       /* Error already printed */
-      return false;
+      goto bail_out;
    }
 
    /* DoSnapShotSet */   
@@ -596,29 +523,32 @@ bool VSSClientGeneric::CreateSnapshots(char* szDriveLetters)
    if (FAILED(hr)) {
       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "DoSnapshotSet");
       errno = b_errno_win32;
-      return false;   
+      goto bail_out;
    }
 
    // Waits for the async operation to finish and checks the result
    if (!WaitAndCheckForAsyncOperation(pAsync2.p)) {
+      /* Error message already printed */
       errno = b_errno_win32;
-      return false;
+      goto bail_out;
    }
    
    /* get latest info about writer status */
    if (!CheckWriterStatus()) {
+      /* Error message already printed */
       errno = b_errno_win32;      /* Error already printed */
-      return false;
+      goto bail_out;
    }
 
    /* query snapshot info */   
    QuerySnapshotSet(m_uidCurrentSnapshotSet);
 
-   SetVSSPathConvert(VSSPathConvert, VSSPathConvertW);
-
    m_bBackupIsInitialized = true;
 
-   return true;
+   ret = true;
+bail_out:
+   V(create_mutex);
+   return ret;
 }
 
 bool VSSClientGeneric::CloseBackup()
@@ -633,17 +563,21 @@ bool VSSClientGeneric::CloseBackup()
       errno = ENOSYS;
       return bRet;
    }
+   /* Create or Delete Snapshot one at a time */
+   P(create_mutex);
+
    CComPtr<IVssAsync>  pAsync;
-
-   SetVSSPathConvert(NULL, NULL);
-
    m_bBackupIsInitialized = false;
 
    hr = pVssObj->BackupComplete(&pAsync.p);
    if (SUCCEEDED(hr)) {
       // Waits for the async operation to finish and checks the result
-      WaitAndCheckForAsyncOperation(pAsync.p);
-      bRet = true;     
+      if (!WaitAndCheckForAsyncOperation(pAsync.p)) {
+         /* Error message already printed */
+         errno = b_errno_win32;
+      } else {
+         bRet = true;
+      }
    } else {
       JmsgVssApiStatus(m_jcr, M_ERROR, hr, "BackupComplete");
       errno = b_errno_win32;
@@ -689,6 +623,7 @@ bool VSSClientGeneric::CloseBackup()
       m_bCoInitializeCalled = false;
    }
 
+   V(create_mutex);
    return bRet;
 }
 
@@ -704,6 +639,7 @@ bool VSSClientGeneric::CloseRestore()
    CComPtr<IVssAsync> pAsync;
 
    if (!pVssObj) {
+      Jmsg(m_jcr, M_FATAL, 0, "No pointer to VssObject or Backup is not Initialized\n");
       errno = ENOSYS;
       return false;
    }
@@ -711,9 +647,14 @@ bool VSSClientGeneric::CloseRestore()
 /* done by plugin now */
    if (SUCCEEDED(hr = pVssObj->PostRestore(&pAsync.p))) {
       // Waits for the async operation to finish and checks the result
-      WaitAndCheckForAsyncOperation(pAsync.p);
+      if (!WaitAndCheckForAsyncOperation(pAsync1.p)) {
+         /* Error message already printed */
+         errno = b_errno_win32;
+         return false;
+      }
       /* get latest info about writer status */
       if (!CheckWriterStatus()) {
+         /* Error message already printed */
          errno = b_errno_win32;
          return false;
       }
@@ -734,8 +675,6 @@ void VSSClientGeneric::QuerySnapshotSet(GUID snapshotSetID)
       return;
    }
 
-   memset(m_szShadowCopyName,0,sizeof (m_szShadowCopyName));
-   
    if (snapshotSetID == GUID_NULL || m_pVssObject == NULL) {
       Jmsg(m_jcr, M_FATAL, 0, "snapshotSetID == NULL or VssObject is NULL.\n");
       errno = ENOSYS;
@@ -765,19 +704,28 @@ void VSSClientGeneric::QuerySnapshotSet(GUID snapshotSetID)
    while (true) {
       // Get the next element
       ULONG ulFetched;
-      hr = (pIEnumSnapshots.p)->Next( 1, &Prop, &ulFetched );
+      hr = (pIEnumSnapshots.p)->Next(1, &Prop, &ulFetched);
 
       // We reached the end of list
-      if (ulFetched == 0)
+      if (ulFetched == 0) {
          break;
+      }
+
+      Dmsg2(DT_VOLUME|50, "Adding %ls => %ls to m_VolumeList\n",
+            Snap.m_pwszOriginalVolumeName, Snap.m_pwszSnapshotDeviceObject);
 
       // Print the shadow copy (if not filtered out)
       if (Snap.m_SnapshotSetId == snapshotSetID)  {
-         for (int ch='A'-'A';ch<='Z'-'A';ch++) {
-            if (wcscmp(Snap.m_pwszOriginalVolumeName, m_wszUniqueVolumeName[ch]) == 0) {       
-               wcsncpy(m_szShadowCopyName[ch],Snap.m_pwszSnapshotDeviceObject, MAX_PATH-1);               
-               break;
+         MTabEntry *elt = (MTabEntry*)m_VolumeList->entries->search(Snap.m_pwszOriginalVolumeName, volume_cmp);
+         if (!elt) {
+            Dmsg1(DT_VOLUME|50, "Unable to find [%ls] in the device list\n", Snap.m_pwszOriginalVolumeName);
+            foreach_rblist(elt, m_VolumeList->entries) {
+               elt->debug_paths();
             }
+            Jmsg(m_jcr, M_WARNING, 0, _("Unable to find volume %ls in the device list\n"), Snap.m_pwszOriginalVolumeName);
+         } else {
+            elt->shadowCopyName = bwcsdup(Snap.m_pwszSnapshotDeviceObject);
+            elt->setInSnapshotSet();
          }
       }
       p_VssFreeSnapshotProperties(&Snap);
@@ -815,6 +763,7 @@ bool VSSClientGeneric::CheckWriterStatus()
 
     // Waits for the async operation to finish and checks the result
     if (!WaitAndCheckForAsyncOperation(pAsync.p)) {
+       /* Error message already printed */
        errno = b_errno_win32;
        return false;
     }
@@ -825,7 +774,7 @@ bool VSSClientGeneric::CheckWriterStatus()
 
     hr = pVssObj->GetWriterStatusCount(&cWriters);
     if (FAILED(hr)) {
-       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "GatherWriterStatusCount");
+       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "GetWriterStatusCount");
        errno = b_errno_win32;
        return false;
     }
