@@ -1,31 +1,21 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2018 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
-
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
- */
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
+   Bacula(R) is a registered trademark of Kern Sibbald.
+*/
 /* 
  * 
  *  Kern Sibbald, August 2007
@@ -45,19 +35,12 @@
  *
  */
 
+#define LOCKMGR_COMPLIANT
 #include "bacula.h"
 #include "win32.h"
 #include <signal.h>
 #include <pthread.h>
 
-#undef  _WIN32_IE
-#ifdef MINGW64
-# define _WIN32_IE 0x0501
-#else
-# define _WIN32_IE 0x0401
-#endif  // MINGW64
-#undef  _WIN32_WINNT
-#define _WIN32_WINNT 0x0501
 #include <commctrl.h>
 
 
@@ -73,7 +56,7 @@ bool GetWindowsVersionString(LPTSTR osbuf, int maxsiz);
 
 
 #define MAX_COMMAND_ARGS 100
-static char *command_args[MAX_COMMAND_ARGS] = {LC_APP_NAME, NULL};
+static char *command_args[MAX_COMMAND_ARGS] = {(char *)LC_APP_NAME, NULL};
 static int num_command_args = 1;
 static pid_t main_pid;
 static pthread_t main_tid;
@@ -96,7 +79,6 @@ int WINAPI WinMain(HINSTANCE Instance, HINSTANCE /*PrevInstance*/, PSTR CmdLine,
    OSVERSIONINFO osversioninfo;
    osversioninfo.dwOSVersionInfoSize = sizeof(osversioninfo);
 
-
    /* Save the application instance and main thread id */
    appInstance = Instance;
    mainthreadId = GetCurrentThreadId();
@@ -366,9 +348,6 @@ int BaculaAppMain()
    }
    WriteConsole(GetStdHandle(STD_OUTPUT_HANDLE), "\r\n", 2, &dwCharsWritten, NULL);
 
-   /* Start up Volume Shadow Copy (only on FD) */
-   VSSInit();
-
    /* Startup networking */
    WSA_Init();
 
@@ -400,7 +379,6 @@ void pause_msg(const char *file, const char *func, int line, const char *msg)
    MessageBox(NULL, buf, "Pause", MB_OK);
 }
 
-#include <windows.h>
 #include <tchar.h>
 #include <stdio.h>
 
@@ -474,8 +452,9 @@ void pause_msg(const char *file, const char *func, int line, const char *msg)
 #endif
 
 /* Unknown value */
+#ifndef VER_SUITE_WH_SERVER
 #define VER_SUITE_WH_SERVER -1
-
+#endif
 
 typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
 typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, PDWORD);
