@@ -1,30 +1,21 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2010 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2018 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
- */
+   Bacula(R) is a registered trademark of Kern Sibbald.
+*/
 /* 
  * 
  *  Kern Sibbald, August 2007
@@ -164,7 +155,7 @@ int baculaServiceMain()
    if (have_service_api) {            /* New style service API */
       /* Tell OS where to dispatch service calls to us */
       SERVICE_TABLE_ENTRY dispatchTable[] = {
-         {APP_NAME, (LPSERVICE_MAIN_FUNCTION)serviceStartCallback},
+         {(char *)APP_NAME, (LPSERVICE_MAIN_FUNCTION)serviceStartCallback},
          {NULL, NULL}};
 
       /* Start the service control dispatcher */
@@ -189,6 +180,7 @@ int baculaServiceMain()
          MessageBox(NULL, _("Registry service not found: Bacula service not started"),
             APP_DESC, MB_OK);
          log_error_message(_("Registry service entry point not found")); 
+         FreeLibrary(kerneldll);         /* free up kernel dll */
          return 1;
       }
       
@@ -273,7 +265,7 @@ int installService(const char *cmdOpts)
               APP_NAME,                       /* Our service name */
               APP_DESC,                       /* Display name */
               SERVICE_ALL_ACCESS,
-              SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS,
+              SERVICE_WIN32_OWN_PROCESS,      /* | SERVICE_INTERACTIVE_PROCESS, */
               SERVICE_AUTO_START,
               SERVICE_ERROR_NORMAL,
               svcmd,                          /* Command string to start the service */
@@ -340,25 +332,23 @@ int installService(const char *cmdOpts)
  */
 int removeService()
 {
-   if (have_service_api) {      /* Newer Windows platform (NT, Win2K, ...) */
-      SC_HANDLE serviceManager, baculaService;
-      int stat = 0;
+   SC_HANDLE serviceManager, baculaService;
+   int stat = 0;
 
+   if (have_service_api) {      /* Newer Windows platforms (NT, Win2K, ...) */
       serviceManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
       if (serviceManager) {
          /* Now get the Bacula service entry */
          baculaService = OpenService(serviceManager, APP_NAME, SERVICE_ALL_ACCESS);
          if (baculaService) {
             SERVICE_STATUS status;
-
             /* If the service is running, stop it */
             if (ControlService(baculaService, SERVICE_CONTROL_STOP, &status)) {
                while(QueryServiceStatus(baculaService, &status)) {
-                  if (status.dwCurrentState == SERVICE_STOP_PENDING) {
-                     sleep(1);
-                  } else {
-                     break;
+                  if (status.dwCurrentState != SERVICE_STOP_PENDING) {
+                     break; 
                   }
+                  sleep(1);
                }
                if (status.dwCurrentState != SERVICE_STOPPED) {
                   if (opt_debug) {
@@ -367,7 +357,6 @@ int removeService()
                   }
                }
             }
-
             if (DeleteService(baculaService)) {
                if (opt_debug) {
                   MessageBox(NULL, _("The Bacula service: " APP_NAME " has been removed"), 
@@ -379,7 +368,6 @@ int removeService()
                stat = 1; /* error */
             }
             CloseServiceHandle(baculaService);
-
          } else {
             if (opt_debug) {
                MessageBox(NULL, _("An existing Bacula service: " APP_NAME " could not be found for "
@@ -389,7 +377,6 @@ int removeService()
          }
          CloseServiceHandle(serviceManager);
          return stat;
-
       } else {
          MessageBox(NULL, _("The service Manager could not be contacted - the Bacula service was not removed"), 
             APP_DESC, MB_ICONEXCLAMATION | MB_OK);
@@ -417,7 +404,6 @@ int removeService()
          RegCloseKey(runservices);
          return 1;
       }
-
       /* Stop any running Bacula */
       if (!stopRunningBacula()) {
          if (opt_debug) {
@@ -427,7 +413,6 @@ int removeService()
          }
          return 0;   /* not really an error */
       }
-
       /* At this point, the service has been removed */
       if (opt_debug) {
          MessageBox(NULL, _("The Bacula service has been removed"), APP_DESC, MB_ICONINFORMATION | MB_OK);
